#include "stdafx.h"
#include <ws2tcpip.h>
#include "TsNetworkSender.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#pragma comment(lib,"Ws2_32.lib")


#ifndef TS_PACKETSIZE
#define TS_PACKETSIZE	(188U)
#endif


CTsNetworkSender::CTsNetworkSender(IEventHandler *pEventHandler)
	: CMediaDecoder(pEventHandler, 1UL, 0UL)
	, m_hSendThread(NULL)
	//, m_hKillEvent(NULL)
	, m_bKillSignal(false)
	, m_SendSize(256)
	, m_SendWait(10)
	, m_bAdjustWait(false)
	, m_bEnableBuffering(false)
	, m_BufferLength(20000)
	, m_pBuffer(NULL)
	, m_BufferUsed(0)
	, m_BufferPos(0)
{
	WSAData WSAData;

	WSAStartup(MAKEWORD(2,0), &WSAData);
}


CTsNetworkSender::~CTsNetworkSender(void)
{
	Close();

	WSACleanup();
}


void CTsNetworkSender::Reset()
{
	CBlockLock Lock(&m_DecoderLock);

	m_BufferUsed = 0;
}


const bool CTsNetworkSender::InputMedia(CMediaData *pMediaData, const DWORD dwInputIndex)
{
	CBlockLock Lock(&m_DecoderLock);

	if (m_BufferUsed == m_BufferLength) {
		return false;
	}

	if (m_bEnableBuffering) {
		::CopyMemory(m_pBuffer + (m_BufferPos + m_BufferUsed) % m_BufferLength * TS_PACKETSIZE,
					 pMediaData->GetData(), TS_PACKETSIZE/*pMediaData->GetSize()*/);
		m_BufferUsed++;
	}

	return true;
}


const bool CTsNetworkSender::Open(const AddressInfo *pList, DWORD Length)
{
	CBlockLock Lock(&m_DecoderLock);

	if (IsOpen())
		return false;

	if (pList == NULL || Length == 0)
		return false;

	const DWORD LocalHost = inet_addr("127.0.0.1");

	for (DWORD i=0; i<Length; i++) {
		SocketInfo Info;
		int Type;

		if (pList[i].Type == SOCKET_UDP) {
			Type = SOCK_DGRAM;
		} else if (pList[i].Type == SOCKET_TCP) {
			Type = SOCK_STREAM;
		} else {
			Close();
			return false;
		}
		Info.Type = pList[i].Type;
		Info.sock = socket(AF_INET, Type, 0);
		if (Info.sock == INVALID_SOCKET) {
			Close();
			return false;
		}

		u_long arg = 1;
		ioctlsocket(Info.sock, FIONBIO, &arg);

		Info.addr.sin_family = AF_INET;
		Info.addr.sin_port = htons(pList[i].Port);
		Info.addr.sin_addr.S_un.S_addr = inet_addr(pList[i].pszAddress);
		if (Info.addr.sin_addr.S_un.S_addr == INADDR_NONE) {
			closesocket(Info.sock);
			Close();
			return false;
		}
		::ZeroMemory(Info.addr.sin_zero, sizeof(Info.addr.sin_zero));

		Info.bConnected = false;
		Info.Event = WSA_INVALID_EVENT;

		if (Type == SOCK_DGRAM)
			setsockopt(Info.sock, IPPROTO_IP, IP_MULTICAST_IF, (const char *)&LocalHost, sizeof(DWORD));
		m_SockList.push_back(Info);
	}

	//m_hKillEvent=::CreateEvent(NULL,FALSE,FALSE,NULL);
	m_bKillSignal=false;
	m_pBuffer=new BYTE[m_BufferLength*TS_PACKETSIZE];
	m_BufferUsed=0;
	m_BufferPos=0;
	m_hSendThread=::CreateThread(NULL,0,SendThread,this,0,NULL);
	if (m_hSendThread==NULL) {
		Close();
		return false;
	}

	return true;
}


const bool CTsNetworkSender::Close()
{
	CBlockLock Lock(&m_DecoderLock);

	if (m_hSendThread) {
		//::SetEvent(m_hKillEvent);
		m_bKillSignal=true;
		if (::WaitForSingleObject(m_hSendThread, 3000) == WAIT_TIMEOUT) {
			::TerminateThread(m_hSendThread, -1);
		}
		::CloseHandle(m_hSendThread);
		m_hSendThread=NULL;
	}
	/*
	if (m_hKillEvent) {
		::CloseHandle(m_hKillEvent);
		m_hKillEvent = NULL;
	}
	*/
	if (m_pBuffer) {
		delete [] m_pBuffer;
		m_pBuffer=NULL;
	}
	m_bEnableBuffering = false;

	for (size_t i=0; i<m_SockList.size(); i++ ) {
		if (m_SockList[i].sock != INVALID_SOCKET) {
			if (m_SockList[i].Event != WSA_INVALID_EVENT) {
				::WSAEventSelect(m_SockList[i].sock, NULL, 0);
				::WSACloseEvent(m_SockList[i].Event);
			}
			closesocket(m_SockList[i].sock);
		}
	}
	m_SockList.clear();

	return true;
}


const bool CTsNetworkSender::IsOpen() const
{
	return m_SockList.size()>0;
}


const bool CTsNetworkSender::SetSendSize(DWORD Size)
{
	if (Size==0)
		return false;
	m_DecoderLock.Lock();
	m_SendSize = Size;
	m_DecoderLock.Unlock();
	return true;
}


const DWORD CTsNetworkSender::GetSendSize() const
{
	return m_SendSize;
}


const bool CTsNetworkSender::SetSendWait(DWORD Wait)
{
	m_SendWait = Wait;
	return true;
}


const DWORD CTsNetworkSender::GetSendWait() const
{
	return m_SendWait;
}


const bool CTsNetworkSender::SetAdjustWait(bool bAdjust)
{
	m_bAdjustWait = bAdjust;
	return true;
}


const bool CTsNetworkSender::GetAdjustWait() const
{
	return m_bAdjustWait;
}


const bool CTsNetworkSender::GetStream(BYTE **ppBuffer,SIZE_T *pBufferLength)
{
	m_DecoderLock.Lock();
	if (m_BufferUsed == 0) {
		m_DecoderLock.Unlock();
		return false;
	}
	SIZE_T Size = min(m_SendSize, m_BufferUsed);
	if (Size > m_BufferLength - m_BufferPos)
		Size = m_BufferLength - m_BufferPos;
	*ppBuffer = m_pBuffer + m_BufferPos * TS_PACKETSIZE;
	*pBufferLength = Size;
	m_DecoderLock.Unlock();
	return true;
}


DWORD CTsNetworkSender::SendThread(LPVOID lpParameter)
{
	CTsNetworkSender *pThis=static_cast<CTsNetworkSender*>(lpParameter);

	int NumTcpSockets = 0;
	for (size_t i = 0 ; i < pThis->m_SockList.size() ; i++) {
		if (pThis->m_SockList[i].Type == SOCKET_TCP)
			NumTcpSockets++;
	}
	if (NumTcpSockets > 0) {
		for (size_t i = 0 ; i < pThis->m_SockList.size() ; i++) {
			if (pThis->m_SockList[i].Type == SOCKET_TCP) {
				WSAEVENT Event = ::WSACreateEvent();

				if (Event != WSA_INVALID_EVENT) {
					if (::WSAEventSelect(pThis->m_SockList[i].sock, Event, FD_CONNECT) == SOCKET_ERROR) {
						::WSACloseEvent(Event);
						Event = WSA_INVALID_EVENT;
					}
				}
				if (Event != WSA_INVALID_EVENT) {
					pThis->m_SockList[i].Event = Event;
				} else {
					u_long arg = 0;
					ioctlsocket(pThis->m_SockList[i].sock, FIONBIO, &arg);
				}
				if (connect(pThis->m_SockList[i].sock,
							(struct sockaddr*)&pThis->m_SockList[i].addr, sizeof(struct sockaddr_in)) == 0) {
					pThis->m_SockList[i].bConnected = true;
					if (Event != WSA_INVALID_EVENT) {
						::WSAEventSelect(pThis->m_SockList[i].sock, NULL, 0);
						::WSACloseEvent(Event);
						pThis->m_SockList[i].Event = WSA_INVALID_EVENT;
					}
				} else {
					int Error = ::WSAGetLastError();
	
					if (Error != WSAEWOULDBLOCK) {
						TRACE(TEXT("CTsNetworkSender::SendThread() connect error %d\n"),
							  Error);
						if (Event != WSA_INVALID_EVENT) {
							::WSAEventSelect(pThis->m_SockList[i].sock, NULL, 0);
							::WSACloseEvent(Event);
							pThis->m_SockList[i].Event = WSA_INVALID_EVENT;
						}
						closesocket(pThis->m_SockList[i].sock);
						pThis->m_SockList[i].sock = INVALID_SOCKET;
					}
				}
			}
		}
		bool bContinue;
		do {
			if (pThis->m_bKillSignal)
				return 0;
			::Sleep(1000);
			bContinue = false;
			for (size_t i = 0 ; i < pThis->m_SockList.size() ; i++) {
				if (pThis->m_SockList[i].Event != WSA_INVALID_EVENT) {
					int Result = ::WSAWaitForMultipleEvents(1, &pThis->m_SockList[i].Event, FALSE, 0, FALSE);

					if (Result == WSA_WAIT_EVENT_0) {
						WSANETWORKEVENTS Events;

						::WSAEnumNetworkEvents(pThis->m_SockList[i].sock,
											   pThis->m_SockList[i].Event, &Events);
						if ((Events.lNetworkEvents & FD_CONNECT) != 0
								&& Events.iErrorCode[FD_CONNECT_BIT] == 0) {
							pThis->m_SockList[i].bConnected = true;
						}
						::WSAEventSelect(pThis->m_SockList[i].sock, NULL, 0);
						::WSACloseEvent(pThis->m_SockList[i].Event);
						pThis->m_SockList[i].Event = WSA_INVALID_EVENT;
					} else if (Result == WSA_WAIT_TIMEOUT) {
						bContinue = true;
					} else {
						::WSAEventSelect(pThis->m_SockList[i].sock, NULL, 0);
						::WSACloseEvent(pThis->m_SockList[i].Event);
						pThis->m_SockList[i].Event = WSA_INVALID_EVENT;
					}
				}
			}
		} while (bContinue);
	}

	CMediaData m_TcpData;
	DWORD Count = 0;

	pThis->m_bEnableBuffering = true;

	//while (::WaitForSingleObject(pThis->m_hKillEvent, 0) == WAIT_TIMEOUT) {
	while (!pThis->m_bKillSignal) {
		BYTE *pSendBuff;
		SIZE_T SendSize;

		if (pThis->GetStream(&pSendBuff, &SendSize)) {
			bool bSetTcpData = false;

			for (size_t i = 0 ; i < pThis->m_SockList.size() ; i++) {
				if (pThis->m_SockList[i].Type == SOCKET_UDP) {
					if (sendto(pThis->m_SockList[i].sock,
							   (const char*)pSendBuff, SendSize * TS_PACKETSIZE,
							   0,
							   (struct sockaddr*)&pThis->m_SockList[i].addr,
							   sizeof(pThis->m_SockList[i].addr)) == 0) {
						/*
						closesocket(pThis->m_SockList[i].sock);
						std::vector<SocketInfo>::iterator itr;
						itr = pThis->m_SockList.begin();
						std::advance(itr, i);
						pThis->m_SockList.erase(itr);
						i--;
						*/
					}
				} else {
					if (pThis->m_SockList[i].bConnected) {
						if (!bSetTcpData) {
							DWORD Size = SendSize * TS_PACKETSIZE;

							m_TcpData.GetBuffer(2 * sizeof(DWORD) + Size);
							m_TcpData.ClearSize();
							m_TcpData.AddData(&Count, sizeof(DWORD));
							m_TcpData.AddData(&Size, sizeof(DWORD));
							m_TcpData.AddData(pSendBuff, Size);
							bSetTcpData = true;
						}
						send(pThis->m_SockList[i].sock,
							 (const char*)m_TcpData.GetData(),
							 m_TcpData.GetSize(),
							 0);
						/*
							EpgDataCap_BonSendTSTCPł͂Count
							CNgĂ邪AႤCc
						*/
						// Count++;
					}
				}
			}
			Count++;
			pThis->m_DecoderLock.Lock();
			pThis->m_BufferPos += SendSize;
			if (pThis->m_BufferPos == pThis->m_BufferLength)
				pThis->m_BufferPos = 0;
			pThis->m_BufferUsed -= SendSize;

			// MԊu(ȂK)
			DWORD Wait = pThis->m_SendWait;
			if (pThis->m_bAdjustWait) {
				if (pThis->m_BufferUsed > pThis->m_BufferLength / 2) {
					Wait -= Wait / 2 * (pThis->m_BufferUsed - pThis->m_BufferLength / 2) / (pThis->m_BufferLength / 2);
				} else {
					Wait += Wait / 2 * (pThis->m_BufferLength / 2 - pThis->m_BufferUsed) / (pThis->m_BufferLength / 2);
				}
			}
			pThis->m_DecoderLock.Unlock();
			::Sleep(Wait);
		} else {
			::Sleep(10);
		}
	}

	return 0;
}
