#include "stdafx.h"
#include <ws2tcpip.h>
#include "TsUdpSender.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#pragma comment(lib,"Ws2_32.lib")


#ifndef TS_PACKETSIZE
#define TS_PACKETSIZE	(188U)
#endif


CTsUdpSender::CTsUdpSender(IEventHandler *pEventHandler)
	: CMediaDecoder(pEventHandler, 1UL, 0UL)
	, m_hSendThread(NULL)
	//, m_hKillEvent(NULL)
	, m_bKillSignal(false)
	, m_SendSize(256)
	, m_SendWait(10)
	, m_BufferLength(20000)
	, m_pBuffer(NULL)
	, m_BufferUsed(0)
	, m_BufferPos(0)
{
	WSAData WSAData;

	WSAStartup(MAKEWORD(2,0), &WSAData);
}


CTsUdpSender::~CTsUdpSender(void)
{
	Close();

	WSACleanup();
}


const bool CTsUdpSender::InputMedia(CMediaData *pMediaData, const DWORD dwInputIndex)
{
	CBlockLock Lock(&m_DecoderLock);

	if (m_BufferUsed==m_BufferLength) {
		return false;
	}

	if (m_SockList.size()>0) {
		::CopyMemory(m_pBuffer+(m_BufferPos+m_BufferUsed)%m_BufferLength*TS_PACKETSIZE,
					 pMediaData->GetData(),TS_PACKETSIZE/*pMediaData->GetSize()*/);
		m_BufferUsed++;
	}

	return true;
}


const bool CTsUdpSender::Open(const AddressInfo *pList, DWORD Length)
{
	CBlockLock Lock(&m_DecoderLock);

	if (IsOpen())
		return false;

	if (pList==NULL || Length==0)
		return false;

	for (DWORD i=0; i<Length; i++) {
		SocketInfo Info;

		Info.sock = socket(AF_INET, SOCK_DGRAM, 0);
		if (Info.sock == INVALID_SOCKET ){
			Close();
			return false;
		}

		u_long arg = 1;
		ioctlsocket(Info.sock, FIONBIO, &arg);

		Info.addr.sin_family = AF_INET;
		Info.addr.sin_port = htons(pList[i].Port);
		Info.addr.sin_addr.S_un.S_addr = inet_addr(pList[i].pszAddress);
		if (Info.addr.sin_addr.S_un.S_addr==INADDR_NONE) {
			closesocket(Info.sock);
			Close();
			return false;
		}
		::ZeroMemory(Info.addr.sin_zero,sizeof(Info.addr.sin_zero));

		DWORD Host = inet_addr("127.0.0.1");
		setsockopt(Info.sock, IPPROTO_IP, IP_MULTICAST_IF, (const char *)&Host, sizeof(DWORD));
		m_SockList.push_back(Info);
	}

	//m_hKillEvent=::CreateEvent(NULL,FALSE,FALSE,NULL);
	m_bKillSignal=false;
	m_pBuffer=new BYTE[m_BufferLength*TS_PACKETSIZE];
	m_BufferUsed=0;
	m_BufferPos=0;
	m_hSendThread=::CreateThread(NULL,0,SendThread,this,0,NULL);
	if (m_hSendThread==NULL) {
		Close();
		return false;
	}

	return true;
}


const bool CTsUdpSender::Close()
{
	CBlockLock Lock(&m_DecoderLock);

	if (m_hSendThread) {
		//::SetEvent(m_hKillEvent);
		m_bKillSignal=true;
		if (::WaitForSingleObject(m_hSendThread, 1000) == WAIT_TIMEOUT) {
			::TerminateThread(m_hSendThread, -1);
		}
		::CloseHandle(m_hSendThread);
		m_hSendThread=NULL;
	}
	/*
	if (m_hKillEvent) {
		::CloseHandle(m_hKillEvent);
		m_hKillEvent = NULL;
	}
	*/
	if (m_pBuffer) {
		delete [] m_pBuffer;
		m_pBuffer=NULL;
	}

	for (size_t i=0; i<m_SockList.size(); i++ ) {
		closesocket(m_SockList[i].sock);
	}
	m_SockList.clear();

	return true;
}


const bool CTsUdpSender::IsOpen() const
{
	return m_SockList.size()>0;
}


const bool CTsUdpSender::SetSendSize(DWORD Size)
{
	if (Size==0)
		return false;
	m_DecoderLock.Lock();
	m_SendSize=Size;
	m_DecoderLock.Unlock();
	return true;
}


const DWORD CTsUdpSender::GetSendSize() const
{
	return m_SendSize;
}


const bool CTsUdpSender::SetSendWait(DWORD Wait)
{
	m_SendWait=Wait;
	return true;
}


const DWORD CTsUdpSender::GetSendWait() const
{
	return m_SendWait;
}


const bool CTsUdpSender::GetStream(BYTE **ppBuffer,SIZE_T *pBufferLength)
{
	m_DecoderLock.Lock();
	if (m_BufferUsed == 0) {
		m_DecoderLock.Unlock();
		return false;
	}
	SIZE_T Size = min(m_SendSize, m_BufferUsed);
	if (Size > m_BufferLength - m_BufferPos)
		Size = m_BufferLength - m_BufferPos;
	*ppBuffer = m_pBuffer+m_BufferPos*TS_PACKETSIZE;
	*pBufferLength = Size;
	m_DecoderLock.Unlock();
	return true;
}


DWORD CTsUdpSender::SendThread(LPVOID lpParameter)
{
	CTsUdpSender *pThis=static_cast<CTsUdpSender*>(lpParameter);

	//while (::WaitForSingleObject(pThis->m_hKillEvent, 0) == WAIT_TIMEOUT) {
	while (!pThis->m_bKillSignal) {
		BYTE *pSendBuff;
		SIZE_T SendSize;

		if (pThis->GetStream(&pSendBuff,&SendSize)) {
			for (size_t i=0; i<pThis->m_SockList.size(); i++) {
				if (sendto(pThis->m_SockList[i].sock,
						   (const char*)pSendBuff, SendSize*TS_PACKETSIZE,
						   0,
						   (struct sockaddr*)&pThis->m_SockList[i].addr,
						   sizeof(pThis->m_SockList[i].addr)) == 0) {
					/*
					closesocket(pThis->m_SockList[i].sock);
					std::vector<SocketInfo>::iterator itr;
					itr = pThis->m_SockList.begin();
					std::advance(itr, i);
					pThis->m_SockList.erase(itr);
					i--;
					*/
				}
			}
			pThis->m_DecoderLock.Lock();
			pThis->m_BufferPos += SendSize;
			if (pThis->m_BufferPos == pThis->m_BufferLength)
				pThis->m_BufferPos = 0;
			pThis->m_BufferUsed -= SendSize;
			pThis->m_DecoderLock.Unlock();
			::Sleep(pThis->m_SendWait);
		} else {
			::Sleep(5);
		}
	}

	return 0;
}
