// NCachedFile.cpp: CNCachedFile NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <new>
#include "NCachedFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CNCachedFile::CNCachedFile()
	: CNFile()
	, m_bIsWritable(false)
	, m_pBuff(NULL)
	, m_BuffSize(0UL)
	, m_DataSize(0UL)
	, m_FilePos(0)
	, m_CurPos(0)
{
}


CNCachedFile::~CNCachedFile()
{
	Close();
}


const bool CNCachedFile::Open(LPCTSTR lpszName, const BYTE bFlags)
{
	return Open(lpszName, bFlags, DEFBUFFSIZE);
}


const bool CNCachedFile::Open(LPCTSTR lpszName, const BYTE bFlags, const DWORD dwBuffSize)
{
	if (IsOpen()) {
		m_LastError = ERROR_BUSY;	// uv\[X͎gpłBv
		return false;
	}

	if ((bFlags & CNFile::CNF_WRITE) && !(bFlags & CNFile::CNF_READ)) {
		// CgLbVL
		m_bIsWritable = true;
	} else if (!(bFlags & CNFile::CNF_WRITE) && (bFlags & CNFile::CNF_READ) && !(bFlags & CNFile::CNF_NEW)) {
		// [hLbVL
		m_bIsWritable = false;
	} else {
		// tȎgݍ킹Ή
		m_LastError = ERROR_INVALID_PARAMETER;
		return false;
	}

	if (dwBuffSize==0) {
		m_LastError = ERROR_INVALID_PARAMETER;
		return false;
	}

	// t@CI[v
	if (!CNFile::Open(lpszName, bFlags))
		return false;

	// obt@m
	m_BuffSize = dwBuffSize;
	if (!m_bIsWritable && (ULONGLONG)dwBuffSize > GetSize()) {
		// ǂݍ݃obt@
		m_BuffSize = (DWORD)GetSize();
	}
	try {
		m_pBuff = new BYTE [m_BuffSize];
	} catch (std::bad_alloc&) {
		Close();
		m_LastError = ERROR_OUTOFMEMORY;
		return false;
	}

	// obt@
	m_DataSize = 0;
	m_FilePos = 0;
	m_CurPos = 0;

	m_LastError = ERROR_SUCCESS;

	return true;
}


const bool CNCachedFile::Close(void)
{
	m_LastError = ERROR_SUCCESS;

	if (IsOpen()) {
		// ݃f[^tbV
		Flush();
		DWORD FlushError = m_LastError;

		CNFile::Close();
		if (FlushError != ERROR_SUCCESS)
			m_LastError = FlushError;

		if (m_pBuff) {
			delete [] m_pBuff;
			m_pBuff = NULL;
		}
	}

	return m_LastError == ERROR_SUCCESS;
}


const DWORD CNCachedFile::Read(void *pBuff, const DWORD dwLen)
{
	// G[
	if (m_bIsWritable) {
		m_LastError = ERROR_INVALID_FUNCTION;
		return 0;
	}

	return CNFile::Read(pBuff, dwLen);
}


const DWORD CNCachedFile::Read(void *pBuff, const DWORD dwLen, const ULONGLONG llPos)
{
	if (m_bIsWritable) {
		m_LastError = ERROR_INVALID_FUNCTION;
		return 0;
	}

	// t@CV[N
	if (!SetPos(llPos))
		return 0;

	return Read(pBuff, dwLen);
}


const bool CNCachedFile::Write(const void *pBuff, const DWORD dwLen)
{
	// t@C
	if (!m_bIsWritable) {
		m_LastError = ERROR_INVALID_FUNCTION;
		return false;
	}

	if (pBuff == NULL || dwLen == 0) {
		m_LastError = ERROR_INVALID_PARAMETER;
		return false;
	}

	// obt@O
	if ((m_BuffSize - m_DataSize) < dwLen) {
		// obt@s
		if (!Flush())
			return false;
	}

	// obt@O
	if ((m_BuffSize - m_DataSize) >= dwLen) {
		::CopyMemory(m_pBuff + m_DataSize, pBuff, dwLen);
		m_DataSize += dwLen;
		if (m_DataSize == m_BuffSize) {
			if (!Flush())
				return false;
		}
	} else {
		if (!CNFile::SetPos(m_CurPos) || !CNFile::Write(pBuff, dwLen))
			return false;
		m_FilePos += dwLen;
	}

	// t@C|WVXV
	m_CurPos += (ULONGLONG)dwLen;

	m_LastError = ERROR_SUCCESS;

	return true;
}


const bool CNCachedFile::Write(const void *pBuff, const DWORD dwLen, const ULONGLONG llPos)
{
	// t@C
	if (!m_bIsWritable) {
		m_LastError = ERROR_INVALID_FUNCTION;
		return false;
	}

	// t@CV[N
	if (!SetPos(llPos))
		return false;

	return Write(pBuff, dwLen);
}


const ULONGLONG CNCachedFile::GetPos(void)
{
	// _Iȃt@C|WVԂ
	return m_bIsWritable ? m_CurPos : CNFile::GetPos();
}


const bool CNCachedFile::SetPos(const ULONGLONG llPos)
{
	// V[NOɃtbV
	if (!Flush())
		return false;

	// V[N
	if (!CNFile::SetPos(llPos))
		return false;

	// t@C|WVXV
	m_FilePos = llPos;
	m_CurPos = llPos;

	return true;
}


const bool CNCachedFile::Flush(void)
{
	m_LastError = ERROR_SUCCESS;

	if (!m_bIsWritable || m_DataSize == 0)
		return true;

	// obt@擪ʒuɏ
	if (!CNFile::SetPos(m_FilePos) || !CNFile::Write(m_pBuff, m_DataSize))
		return false;
	m_FilePos += m_DataSize;

	// obt@TCYNA
	m_DataSize = 0;

	return true;
}
