#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H


#include "BasicWindow.h"
#include "ChannelManager.h"


class CMainWindow : public CBasicWindow {
	bool m_fAlwaysOnTop;
	bool m_fStandby;
	bool m_fStandbyInit;
	bool m_fMinimizeInit;
	bool m_fSrcFilterReleased;
	CChannelSpec m_RestoreChannelSpec;
	bool m_fExitOnRecordingStop;
	POINT m_ptDragStartPos;
	RECT m_rcDragStart;
	bool m_fClosing;
	HFONT m_hfontCaption;
	bool OnCreate();
	void OnCommand(HWND hwnd,int id,HWND hwndCtl,UINT codeNotify);
	void OnTimer(HWND hwnd,UINT id);
	BOOL OnAppCommand(HWND hwnd,HWND hwndFrom,int nCommand,UINT uDevice,DWORD dwKeys);
	void CloseTuner();
	static CMainWindow *GetThis(HWND hwnd);
	static LRESULT CALLBACK WndProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam);
	static BOOL CALLBACK AdjustPosProc(HWND hwnd,LPARAM lParam);
public:
	enum {
		TIMER_ID_UPDATE=1,
		TIMER_ID_WHEELCHANNELCHANGE,
		TIMER_ID_WHEELCHANNELCHANGE_DONE,
		TIMER_ID_INFOTIP
	};
	CMainWindow();
	~CMainWindow();
	bool Create(HWND hwndParent,DWORD Style,DWORD ExStyle=0,int ID=0);
	bool Show(int CmdShow);
	HWND GetVideoHostWindow() const;
	int ShowMessage(LPCTSTR pszText,LPCTSTR pszCaption=NULL,UINT Type=MB_OK | MB_ICONEXCLAMATION) const;
	enum {
		MESSAGE_ICON_NONE,
		MESSAGE_ICON_INFO,
		MESSAGE_ICON_WARNING,
		MESSAGE_ICON_ERROR
	};
	bool PopupMessage(LPCTSTR pszText,LPCTSTR pszTitle=NULL,int Icon=MESSAGE_ICON_NONE,DWORD TimeOut=5000);
	bool PopupErrorMessage(LPCTSTR pszText,DWORD TimeOut=5000) {
		return PopupMessage(pszText,TEXT("Error"),MESSAGE_ICON_ERROR,TimeOut);
	}
	void AdjustPos();
	void SetAlwaysOnTop(bool fTop);
	bool GetAlwaysOnTop() const { return m_fAlwaysOnTop; }
	void SetWindowVisible();
	bool SetStandby(bool fStandby);
	bool GetStandby() const { return m_fStandby; }
	bool InitStandby();
	bool InitMinimize();
	bool IsMinimizeToTray() const;
	bool OpenTuner();
	bool ConfirmExit();
	bool ConfirmChannelChange();
	void OnMouseWheel(WPARAM wParam,LPARAM lParam);
	void PopupMenu(const POINT *pPos=NULL);
	void SendCommand(int Command) { OnCommand(m_hwnd,Command,NULL,0); }
	void PostCommand(int Command) { PostMessage(WM_COMMAND,Command,0); }
	bool CommandLineRecord(LPCTSTR pszFileName,DWORD Delay,DWORD Duration);
	bool GetExitOnRecordingStop() const { return m_fExitOnRecordingStop; }
	void SetExitOnRecordingStop(bool fExit) { m_fExitOnRecordingStop=fExit; }
	static bool Initialize();
};


#endif
