#include "stdafx.h"
#include <shlwapi.h>
#include "RecTest.h"
#include "AppMain.h"
#include "UdpOptions.h"
#include "DialogUtil.h"
#include "resource.h"




CUdpOptions::CUdpOptions()
{
	GetAppClass().GetAppDirectory(m_szViewProgramPath);
	::PathAppend(m_szViewProgramPath,TEXT("TVTest.exe"));
	::lstrcpy(m_szViewProgramParameter,TEXT("/d BonDriver_UDP.dll /nd /port %port%"));
	m_SocketType=CTsNetworkSender::SOCKET_UDP;
	::lstrcpy(m_szAddress,TEXT("127.0.0.1"));
	m_Port=1235;
	m_hPortMutex=NULL;
	m_RealPort=0;
	m_fSendCurServiceOnly=false;
	m_fSendSubtitle=true;
	m_fSendDataCarrousel=true;
}


CUdpOptions::~CUdpOptions()
{
	if (m_hPortMutex) {
		::ReleaseMutex(m_hPortMutex);
		::CloseHandle(m_hPortMutex);
	}
}


bool CUdpOptions::Read(CSettings *pSettings)
{
	pSettings->Read(TEXT("ViewerPath"),m_szViewProgramPath,lengthof(m_szViewProgramPath));
	pSettings->Read(TEXT("ViewerParameter"),m_szViewProgramParameter,lengthof(m_szViewProgramParameter));
	int Protocol;
	if (pSettings->Read(TEXT("NetworkProtocol"),&Protocol)
			&& Protocol>=0 && Protocol<=1)
		m_SocketType=(CTsNetworkSender::SocketType)Protocol;
	pSettings->Read(TEXT("UdpSendAddress"),m_szAddress,lengthof(m_szAddress));
	pSettings->Read(TEXT("UdpSendPort"),&m_Port);
	CTsNetworkSender *pNetworkSender=&GetAppClass().GetCoreEngine()->m_DtvEngine.m_TsNetworkSender;
	unsigned int SendSize;
	if (pSettings->Read(TEXT("UdpSendSize"),&SendSize))
		pNetworkSender->SetSendSize(SendSize);
	unsigned int SendWait;
	if (pSettings->Read(TEXT("UdpSendWait"),&SendWait))
		pNetworkSender->SetSendWait(SendWait);
	bool fAdjustWait;
	if (pSettings->Read(TEXT("UdpAdjustWait"),&fAdjustWait))
		pNetworkSender->SetAdjustWait(fAdjustWait);
	pSettings->Read(TEXT("UdpSendCurServiceOnly"),&m_fSendCurServiceOnly);
	pSettings->Read(TEXT("NetworkSendSubtitle"),&m_fSendSubtitle);
	pSettings->Read(TEXT("NetworkSendDataCarrousel"),&m_fSendDataCarrousel);
	return true;
}


bool CUdpOptions::Write(CSettings *pSettings) const
{
	pSettings->Write(TEXT("ViewerPath"),m_szViewProgramPath);
	pSettings->Write(TEXT("ViewerParameter"),m_szViewProgramParameter);
	pSettings->Write(TEXT("NetworkProtocol"),(int)m_SocketType);
	pSettings->Write(TEXT("UdpSendAddress"),m_szAddress);
	pSettings->Write(TEXT("UdpSendPort"),m_Port);
	const CTsNetworkSender *pNetworkSender=&GetAppClass().GetCoreEngine()->m_DtvEngine.m_TsNetworkSender;
	pSettings->Write(TEXT("UdpSendSize"),(unsigned int)pNetworkSender->GetSendSize());
	pSettings->Write(TEXT("UdpSendWait"),(unsigned int)pNetworkSender->GetSendWait());
	pSettings->Write(TEXT("UdpAdjustWait"),pNetworkSender->GetAdjustWait());
	pSettings->Write(TEXT("UdpSendCurServiceOnly"),m_fSendCurServiceOnly);
	pSettings->Write(TEXT("NetworkSendSubtitle"),m_fSendSubtitle);
	pSettings->Write(TEXT("NetworkSendDataCarrousel"),m_fSendDataCarrousel);
	return true;
}


bool CUdpOptions::ExecuteViewer() const
{
	TCHAR szCommandLine[MAX_PATH*2];
	int i;
	LPCTSTR p;

	if (m_szViewProgramPath[0]=='\0')
		return false;

	::lstrcpy(szCommandLine,m_szViewProgramPath);
	::PathQuoteSpaces(szCommandLine);
	if (m_szViewProgramParameter[0]!='\0') {
		p=m_szViewProgramParameter;
		i=::lstrlen(szCommandLine);
		szCommandLine[i++]=' ';
		do {
			if (*p=='%') {
				if (StrCmpNI(p,TEXT("%port%"),6)==0) {
					i+=::wsprintf(szCommandLine+i,TEXT("%d"),
								  m_RealPort!=0?m_RealPort:m_Port);
					p+=6;
				} else {
					szCommandLine[i++]=*p;
					do {
						p++;
						szCommandLine[i++]=*p;
					} while (*p!='%' && *p!='\0');
					if (*p!='\0')
						p++;
				}
			} else {
				szCommandLine[i++]=*p++;
			}
		} while (*p!='\0');
		szCommandLine[i]='\0';
	}
	TRACE(TEXT("Execute viewer : %s\n"),szCommandLine);
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	::ZeroMemory(&si,sizeof(si));
	si.cb=sizeof(si);
	::ZeroMemory(&pi,sizeof(pi));
	if (!::CreateProcess(NULL,szCommandLine,NULL,NULL,FALSE,0,NULL,NULL,&si,&pi))
		return false;
	::CloseHandle(pi.hProcess);
	::CloseHandle(pi.hThread);
	return true;
}


bool CUdpOptions::StartNetworkSend(int Protocol,unsigned int Port)
{
	CTsNetworkSender::AddressInfo AddressInfo;
	char szAddr[64];

	if (m_szAddress[0]=='\0' || m_Port==0)
		return false;

	if (m_hPortMutex!=NULL) {
		::ReleaseMutex(m_hPortMutex);
		::CloseHandle(m_hPortMutex);
		m_hPortMutex=NULL;
	}

	GetAppClass().GetCoreEngine()->m_DtvEngine.SetSendCurServiceOnly(m_fSendCurServiceOnly);

	TCHAR szName[64];
	if (Port==0)
		Port=m_Port;
	while (true) {
		::wsprintf(szName,APP_NAME TEXT(" UDP Port %u"),Port);
		HANDLE hMutex=::OpenMutex(MUTEX_ALL_ACCESS,FALSE,szName);
		if (hMutex==NULL)
			break;
		::CloseHandle(hMutex);
		Port++;
	}
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	::ZeroMemory(&sd,sizeof(sd));
	::InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	::SetSecurityDescriptorDacl(&sd,TRUE,NULL,FALSE);
	::ZeroMemory(&sa,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;
	m_hPortMutex=::CreateMutex(&sa,TRUE,szName);

	::WideCharToMultiByte(CP_ACP,0,m_szAddress,-1,szAddr,lengthof(szAddr),NULL,NULL);
	if (Protocol==PROTOCOL_DEFAULT) {
		AddressInfo.Type=m_SocketType;
	} else {
		AddressInfo.Type=Protocol==PROTOCOL_UDP?
			CTsNetworkSender::SOCKET_UDP:CTsNetworkSender::SOCKET_TCP;
	}
	AddressInfo.pszAddress=szAddr;
	AddressInfo.Port=Port;
	if (!GetAppClass().GetCoreEngine()->m_DtvEngine.StartNetworkSend(&AddressInfo,1))
		return false;
	m_RealPort=Port;
	return true;
}


bool CUdpOptions::StopNetworkSend()
{
	if (m_RealPort==0)
		return true;
	if (m_hPortMutex!=NULL) {
		::ReleaseMutex(m_hPortMutex);
		::CloseHandle(m_hPortMutex);
		m_hPortMutex=NULL;
	}
	m_RealPort=0;
	return GetAppClass().GetCoreEngine()->m_DtvEngine.StopNetworkSend();
}


bool CUdpOptions::IsNetworkSending() const
{
	return m_RealPort>0;
}


int CUdpOptions::GetProtocol() const
{
	if (m_SocketType==CTsNetworkSender::SOCKET_UDP)
		return PROTOCOL_UDP;
	return PROTOCOL_TCP;
}


CUdpOptions *CUdpOptions::GetThis(HWND hDlg)
{
	return static_cast<CUdpOptions*>(GetOptions(hDlg));
}


BOOL CALLBACK CUdpOptions::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		{
			CUdpOptions *pThis=static_cast<CUdpOptions*>(OnInitDialog(hDlg,lParam));

			::SetDlgItemText(hDlg,IDC_UDPOPTIONS_VIEWERPATH,pThis->m_szViewProgramPath);
			::SetDlgItemText(hDlg,IDC_UDPOPTIONS_VIEWERPARAMETER,pThis->m_szViewProgramParameter);
			CheckRadioButton(hDlg,IDC_NETWORKOPTIONS_PROTOCOL_UDP,
								  IDC_NETWORKOPTIONS_PROTOCOL_TCP,
							 pThis->m_SocketType==CTsNetworkSender::SOCKET_UDP?
								  IDC_NETWORKOPTIONS_PROTOCOL_UDP:
								  IDC_NETWORKOPTIONS_PROTOCOL_TCP);
			::SetDlgItemText(hDlg,IDC_UDPOPTIONS_ADDRESS,pThis->m_szAddress);
			::SetDlgItemInt(hDlg,IDC_UDPOPTIONS_PORT,pThis->m_Port,FALSE);
			::SendDlgItemMessage(hDlg,IDC_UDPOPTIONS_PORT_UD,UDM_SETRANGE32,
								 1,0xFFFF);
			const CTsNetworkSender *pNetworkSender=&GetAppClass().GetCoreEngine()->m_DtvEngine.m_TsNetworkSender;
			::SetDlgItemInt(hDlg,IDC_UDPOPTIONS_SENDSIZE,pNetworkSender->GetSendSize(),FALSE);
			::SendDlgItemMessage(hDlg,IDC_UDPOPTIONS_SENDSIZE_UD,UDM_SETRANGE32,
								 1,0xFFFF);
			::SetDlgItemInt(hDlg,IDC_UDPOPTIONS_SENDWAIT,pNetworkSender->GetSendWait(),FALSE);
			::SendDlgItemMessage(hDlg,IDC_UDPOPTIONS_SENDWAIT_UD,UDM_SETRANGE32,
								 0,0xFFFF);
			DlgCheckBox_Check(hDlg,IDC_NETWORKOPTIONS_ADJUSTWAIT,pNetworkSender->GetAdjustWait());
			DlgCheckBox_Check(hDlg,IDC_UDPOPTIONS_SENDCURSERVICEONLY,pThis->m_fSendCurServiceOnly);
			DlgCheckBox_Check(hDlg,IDC_NETWORKOPTIONS_SENDSUBTITLE,pThis->m_fSendSubtitle);
			DlgCheckBox_Check(hDlg,IDC_NETWORKOPTIONS_SENDDATACARROUSEL,pThis->m_fSendDataCarrousel);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_UDPOPTIONS_VIEWERPATH_BROWSE:
			{
				OPENFILENAME ofn;
				TCHAR szFileName[MAX_PATH],szInitialDir[MAX_PATH];

				::GetDlgItemText(hDlg,IDC_UDPOPTIONS_VIEWERPATH,
								 szFileName,lengthof(szFileName));
				ofn.lStructSize=sizeof(OPENFILENAME);
				ofn.hwndOwner=hDlg;
				ofn.lpstrFilter=TEXT("st@C(*.exe)\0*.exe\0")
								TEXT("ׂẴt@C\0*.*\0");
				ofn.lpstrCustomFilter=NULL;
				ofn.nFilterIndex=1;
				ofn.lpstrFile=szFileName;
				ofn.nMaxFile=lengthof(szFileName);
				ofn.lpstrFileTitle=NULL;
				if (szFileName[0]=='\0' || ::PathIsFileSpec(szFileName)) {
					::GetModuleFileName(NULL,szInitialDir,lengthof(szInitialDir));
					*(::PathFindFileName(szInitialDir)-1)='\0';
					ofn.lpstrInitialDir=szInitialDir;
				} else
					ofn.lpstrInitialDir=NULL;
				ofn.lpstrTitle=TEXT("vO");
				ofn.Flags=OFN_EXPLORER | OFN_HIDEREADONLY | OFN_FILEMUSTEXIST;
				ofn.lpstrDefExt=TEXT("exe");
#if _WIN32_WINNT>=0x500
				ofn.pvReserved=NULL;
				ofn.dwReserved=0;
				ofn.FlagsEx=0;
#endif
				if (::GetOpenFileName(&ofn))
					::SetDlgItemText(hDlg,IDC_UDPOPTIONS_VIEWERPATH,szFileName);
			}
			return TRUE;
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case PSN_APPLY:
			{
				CUdpOptions *pThis=GetThis(hDlg);
				CTsNetworkSender *pNetworkSender=&GetAppClass().GetCoreEngine()->m_DtvEngine.m_TsNetworkSender;

				pThis->m_SocketType=
					::IsDlgButtonChecked(hDlg,IDC_NETWORKOPTIONS_PROTOCOL_UDP)?
						CTsNetworkSender::SOCKET_UDP:CTsNetworkSender::SOCKET_TCP;
				::GetDlgItemText(hDlg,IDC_UDPOPTIONS_VIEWERPATH,
								 pThis->m_szViewProgramPath,MAX_PATH);
				::GetDlgItemText(hDlg,IDC_UDPOPTIONS_VIEWERPARAMETER,
								 pThis->m_szViewProgramParameter,lengthof(pThis->m_szViewProgramParameter));
				TCHAR szAddress[64];
				::GetDlgItemText(hDlg,IDC_UDPOPTIONS_ADDRESS,szAddress,lengthof(szAddress));
				unsigned int Port=::GetDlgItemInt(hDlg,IDC_UDPOPTIONS_PORT,NULL,FALSE);
				if (::lstrcmp(szAddress,pThis->m_szAddress)!=0
						|| Port!=pThis->m_Port) {
					::lstrcpy(pThis->m_szAddress,szAddress);
					pThis->m_Port=Port;
					if (szAddress[0]!='\0' && Port!=0 && pNetworkSender->IsOpen())
						pThis->StartNetworkSend();
				}
				unsigned int SendSize=::GetDlgItemInt(hDlg,IDC_UDPOPTIONS_SENDSIZE,NULL,FALSE);
				pNetworkSender->SetSendSize(SendSize);
				pNetworkSender->SetSendWait(::GetDlgItemInt(hDlg,IDC_UDPOPTIONS_SENDWAIT,NULL,FALSE));
				pNetworkSender->SetAdjustWait(DlgCheckBox_IsChecked(hDlg,IDC_NETWORKOPTIONS_ADJUSTWAIT));
				pThis->m_fSendCurServiceOnly=
					DlgCheckBox_IsChecked(hDlg,IDC_UDPOPTIONS_SENDCURSERVICEONLY);
				pThis->m_fSendSubtitle=
					DlgCheckBox_IsChecked(hDlg,IDC_NETWORKOPTIONS_SENDSUBTITLE);
				pThis->m_fSendDataCarrousel=
					DlgCheckBox_IsChecked(hDlg,IDC_NETWORKOPTIONS_SENDDATACARROUSEL);
				DWORD Stream=CTsSelector::STREAM_ALL;
				if (!pThis->m_fSendSubtitle)
					Stream^=CTsSelector::STREAM_SUBTITLE;
				if (!pThis->m_fSendDataCarrousel)
					Stream^=CTsSelector::STREAM_DATACARROUSEL;
				GetAppClass().GetCoreEngine()->m_DtvEngine.SetSendCurServiceOnly(pThis->m_fSendCurServiceOnly,Stream);
			}
			break;
		}
		break;

	case WM_DESTROY:
		{
			CUdpOptions *pThis=GetThis(hDlg);

			pThis->OnDestroy();
		}
		return TRUE;
	}
	return FALSE;
}
